/*
// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.*/

package org.opengroup.osdu.notification.provider.azure.util;

import com.azure.security.keyvault.secrets.SecretClient;
import com.azure.security.keyvault.secrets.models.KeyVaultSecret;
import org.opengroup.osdu.notification.provider.interfaces.IAppProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;

@Configuration
public class AppProperties implements IAppProperties {

    @Value("${app.entitlements}")
    private String authorizeAPI;

    @Value("${app.register}")
    private String registerAPI;

    @Value("${app.expireTime}")
    private String expireTime;

    @Value("${app.maxCacheSize}")
    private String maxCacheSize;

    @Value("${aad.oboApi}")
    private String aadOboAPI;

    @Value("${aad.eventGridId}")
    private String eventGridId;

    @Autowired
    private SecretClient secretClient;

    private String authURL;

    private String authClientID;

    private String authClientSecret;

    public String getAadClientID() {
        return aadOboAPI;
    }

    public String getEventGridId() {
        return eventGridId;
    }

    public String getAuthorizeAPI() {
        return this.authorizeAPI;
    }

    public String getRegisterAPI() {
        return this.registerAPI;
    }

    public String getAuthClientSecret() {
        if(this.authClientSecret == null) {
            this.authClientSecret = getKeyVaultSecret(this.secretClient, "app-dev-sp-password");
        }

        return this.authClientSecret;
    }

    public String getAuthClientID() {
        if(this.authClientID == null) {
            this.authClientID = getKeyVaultSecret(this.secretClient, "app-dev-sp-username");
        }

        return this.authClientID;
    }

    public String getAuthURL() {
        if(this.authURL == null) {
            String urlFormat = "https://login.microsoftonline.com/%s/oauth2/token/";
            String tenant = getKeyVaultSecret(this.secretClient, "app-dev-sp-tenant-id");
            this.authURL = String.format(urlFormat, tenant);
        }

        return this.authURL;
    }

    public String getMaxCacheSize() {
        return maxCacheSize;
    }

    private String getKeyVaultSecret(SecretClient kv, String secretName) {
        KeyVaultSecret secret = kv.getSecret(secretName);
        if (secret == null) {
            throw new IllegalStateException(String.format("No secret found with name %s", secretName));
        }

        String secretValue = secret.getValue();
        if (secretValue == null) {
            throw new IllegalStateException(String.format(
                    "Secret unexpectedly missing from KeyVault response for secret with name %s", secretName));
        }

        return secretValue;
    }
}
